#! /usr/bin/python
# -*- coding: utf8 -*-
# flake8: noqa E741

import datetime

from ldap_inp import L
from ldap_inp.ldapobject import escape_characters
from ldap_inp.utf_tools import to_alphanumeric

INVALIDES_DANS_NOM = frozenset("0123456789")

# Definitions des correspondances nom=>filière
RACC_FIL = {
    "ge": "ou=gea,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "gea": "ou=gea,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "genieelectriqueetautomatique": "ou=gea,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "en": "ou=en,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "electronique": "ou=en,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "info": "ou=in,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "informatique": "ou=in,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "ima": "ou=in,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "hy": "ou=hy,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "hmf": "ou=hy,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "hydro": "ou=hy,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "hydraulique": "ou=hy,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "ahy": "ou=ahy,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "ahydro": "ou=ahy,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "apprentishydro": "ou=ahy,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "apprentishydraulique": "ou=ahy,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "tr": "ou=tr,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "telecomsreseaux": "ou=tr,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "ir": "ou=ir,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "informatiqueetreseaux": "ou=ir,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "ege": "ou=ege,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "engea": "ou=ege,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "electroniqueetgenieelectrique": "ou=ege,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "mastere": "ou=mast,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "biotech": "ou=biotech,ou=filieres,o=ensat,dc=etu-inpt,dc=fr",
    "biotechnologieaméliorationetprotectiondesplantes": "ou=biotech,ou=filieres,o=ensat,dc=etu-inpt,dc=fr",
    "fcommune": "ou=c,ou=filieres,o=ensat,dc=etu-inpt,dc=fr",
    "commune": "ou=c,ou=filieres,o=ensat,dc=etu-inpt,dc=fr",
    "gestion": "ou=geca,ou=filieres,o=ensat,dc=etu-inpt,dc=fr",
    "gestioneconomieetcommunicationdanslesagroactivites": "ou=geca,ou=filieres,o=ensat,dc=etu-inpt,dc=fr",
    "iaa": "ou=iaa,ou=filieres,o=ensat,dc=etu-inpt,dc=fr",
    "industriesagricolesetalimentairesinnovationetqualite": "ou=iaa,ou=filieres,o=ensat,dc=etu-inpt,dc=fr",
    "iaege": "ou=iaege,ou=filieres,o=ensat,dc=etu-inpt,dc=fr",
    "ingenierieagronomiqueenvironnementetgestiondelespace": "ou=iaege,ou=filieres,o=ensat,dc=etu-inpt,dc=fr",
    "pafqp": "ou=pafqp,ou=filieres,o=ensat,dc=etu-inpt,dc=fr",
    "productionsanimalesfileresetqualitedesproduits": "ou=pafqp,ou=filieres,o=ensat,dc=etu-inpt,dc=fr",
    "pvva": "ou=pvva,ou=filieres,o=ensat,dc=etu-inpt,dc=fr",
    "productionsvegetalesetvalorisationdesagroressources": "ou=pvva,ou=filieres,o=ensat,dc=etu-inpt,dc=fr",
    "chimie": "ou=ch,ou=filieres,o=a7,dc=etu-inpt,dc=fr",
    "gc": "ou=gc,ou=filieres,o=a7,dc=etu-inpt,dc=fr",
    "geniechimique": "ou=gc,ou=filieres,o=a7,dc=etu-inpt,dc=fr",
    "gpi": "ou=gpi,ou=filieres,o=a7,dc=etu-inpt,dc=fr",
    "geniedesprocedesetinformatique": "ou=gpi,ou=filieres,o=a7,dc=etu-inpt,dc=fr",
    "gi": "ou=gi,ou=filieres,o=v,dc=etu-inpt,dc=fr",
    "genieindustriel": "ou=gi,ou=filieres,o=v,dc=etu-inpt,dc=fr",
    "la": "ou=la,ou=filieres,o=a7,dc=etu-inpt,dc=fr",
    "licence agro": "ou=la,ou=filieres,o=a7,dc=etu-inpt,dc=fr",  # outdated,n'existe plus
    "materiaux": "ou=imat,ou=filieres,o=a7,dc=etu-inpt,dc=fr",
    "mep": "ou=mep,ou=filieres,o=a7,dc=etu-inpt,dc=fr",
    "materiauxetprocedes": "ou=mep,ou=filieres,o=a7,dc=etu-inpt,dc=fr",
    "fiva": "ou=fiva,ou=filieres,o=a7,dc=etu-inpt,dc=fr",
    "apprentissage": "ou=fiva,ou=filieres,o=a7,dc=etu-inpt,dc=fr",
    "sn": "ou=sdn,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "eeea": "ou=eeea,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "3ea": "ou=eeea,ou=filieres,o=n7,dc=etu-inpt,dc=fr",
    "enm": "ou=ingenieur,ou=filieres,o=enm,dc=etu-inpt,dc=fr",
}


def extend_filiere(mot):
    try:
        return RACC_FIL[to_alphanumeric(mot)]
    except KeyError:
        return None


def extend_genre(mot):
    if mot == "fille":
        return "2"
    elif mot.encode("utf8") == "garçon":
        return "1"
    else:
        return None


def extend_promo(mot):
    if mot[:-1].isdigit() and mot[-1].lower() == "a":
        an = int(mot[:-1])
        today = datetime.date.today()
        if today < datetime.date(year=today.year, month=9, day=1):
            # On est dans la periode Printemps: promo = annee + 3 - an demande
            return str(today.year + 3 - an)
        else:
            # On est dans la periode Automne: promo = annee + 4 - an demande
            return str(today.year + 4 - an)

    return mot


def combi(liste, n):
    """Renvoie la liste des combinaisons (tuples) de n éléments
    parmi les élèments de la liste
    combi([1, 2, 3], 2) = [(1, 2), (1, 3), (2, 3)]
    """
    if n < 1 or n > len(liste):
        return []

    if n == 1:
        return [(e,) for e in liste]

    l = []
    for k, e in enumerate(liste[:-1]):
        c = combi(liste[k + 1 :], n - 1)
        for f in c:
            l.append((e,) + f)
    return l


def perm(l):
    """Renvoie la liste des permutations (listes)
    formées à partir des élèments de la liste
    perm([1, 2, 3]) = [[1, 2, 3], [1, 3, 2], ..., [3, 2, 1]]
    """
    sz = len(l)
    if sz <= 1:
        return [l]
    return [p[:i] + [l[0]] + p[i:] for i in range(sz) for p in perm(l[1:])]


def gen_filterldap(linput, champs):
    if len(linput) > len(champs):
        return None

    combic = combi(champs, len(linput))

    filtre = None
    for li in perm(linput):
        for c in combic:
            dic_filtre = {}
            filtre_ok = True
            for k, i in zip(c, li):
                i = escape_characters(i, delete_chars=True)
                if k == "filiere":
                    filiere = extend_filiere(i)
                    if not filiere:
                        filtre_ok = False
                    dic_filtre[k] = filiere
                elif k == "genre":
                    genre = extend_genre(i)
                    if not genre:
                        filtre_ok = False
                    dic_filtre[k] = genre
                elif k == "promo":
                    promo = extend_promo(i)
                    if not promo.isdigit():
                        filtre_ok = False
                    dic_filtre[k] = promo

                else:
                    # Cas ou on a snSearch ou givenNameSearch : simplification
                    # du nom
                    # if not INVALIDES_DANS_NOM.isdisjoint(i):
                    #     filtre_ok = False

                    if k.endswith("Search"):
                        i = to_alphanumeric(i)
                    if not i or len(i) < 3:
                        filtre_ok = False
                    dic_filtre[k + "__matching"] = "*" + i + "*"

            if filtre_ok:
                filtre |= L(**dic_filtre)

    # On rajoute l'uid
    if len(linput) == 1:
        filtre |= L(uid=linput[0])

    return filtre


MAX_WORDS = 10


def combispace(liste):
    """Renvoie les 'combi' de n str
    Si n vaut 'pipo pouet toto' alors
    combispace(n) = [['pipo', 'pouet', 'toto'],
                    ['pipo pouet', 'toto'],
                    ['pipo', 'pouet toto'],
                    ['pipo pouet toto']]
    """
    if len(liste) >= MAX_WORDS:
        return None

    taille = len(liste)
    if taille > 1:
        final = []
        for i in range(1, taille):
            temp = combispace(liste[i:])
            longueur = len(temp)
            for j in range(0, longueur):
                temp[j].append(" ".join(liste[:i]))
                final.append(temp[j])
        final.append([" ".join(liste)])
        return final
    else:
        return [liste]


if __name__ == "__main__":
    print(gen_filterldap("Alexandre Chataignon".split(), ("nom", "prenom", "nickname")))
    inp = "1 2 3 4 5 6 7 8 9".split()
    t = 0
    for c in combispace(inp):
        if len(c) <= 5:
            t += 1
    print(t)
